@@ -127,6 +127,8 @@
 #define AFS_AG_ENTRY_CB_TAG          'GAFA'
 #define AFS_PROCESS_AG_CB_TAG        'APFA'
 #define AFS_BYTERANGE_TAG            '_RBA'
+#define AFS_HARDLINK_REQUEST_TAG     'LFFA'
+
 #define __Enter
 
 #define try_return(S) { S; goto try_exit; }
@@ -237,5 +239,13 @@ static inline void AFSBreakPoint() {
 
 #define AFS_REDIR_LIBRARY_SERVICE_ENTRY     L"\\Registry\\Machine\\System\\CurrentControlSet\\Services\\AFSLibrary"
 
+//
+// Common Device flags
+//
+
+#define AFS_DEVICE_FLAG_HIDE_DOT_NAMES          0x00000001
+#define AFS_DEVICE_FLAG_REDIRECTOR_SHUTDOWN     0x00000002
+#define AFS_DEVICE_FLAG_DISABLE_SHORTNAMES      0x00000004
+
 #endif
 
