@@ -63,6 +63,7 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
     AFSCcb *pCcb = NULL;
     AFSObjectInfoCB *pObjectInfo = NULL;
     AFSDirectoryCB *pDirCB = NULL;
+    LONG lCount;
 
     __try
     {
@@ -167,9 +168,7 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     ntStatus = STATUS_SUCCESS;
                 }
 
-                ASSERT( pDirCB->OpenReferenceCount > 0);
-
-                InterlockedDecrement( &pDirCB->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pDirCB->DirOpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
@@ -177,7 +176,9 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                               &pDirCB->NameInformation.FileName,
                               pDirCB,
                               pCcb,
-                              pDirCB->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 //
                 // If this is not the root then decrement the open child reference count
@@ -198,15 +199,15 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
 
                 AFSReleaseResource( &pFcb->NPFcb->Resource);
 
-                ASSERT( pFcb->OpenReferenceCount != 0);
-
-                InterlockedDecrement( &pFcb->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pFcb->OpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
                               "AFSClose (IOCtl) Decrement count on Fcb %08lX Cnt %d\n",
                               pFcb,
-                              pFcb->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 break;
             }
@@ -248,9 +249,7 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     ntStatus = STATUS_SUCCESS;
                 }
 
-                ASSERT( pDirCB->OpenReferenceCount > 0);
-
-                InterlockedDecrement( &pDirCB->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pDirCB->DirOpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
@@ -258,19 +257,21 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                               &pDirCB->NameInformation.FileName,
                               pDirCB,
                               pCcb,
-                              pDirCB->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 AFSReleaseResource( &pFcb->NPFcb->Resource);
 
-                ASSERT( pFcb->OpenReferenceCount > 0);
-
-                InterlockedDecrement( &pFcb->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pFcb->OpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
                               "AFSClose (RootAll) Decrement count on Fcb %08lX Cnt %d\n",
                               pFcb,
-                              pFcb->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 break;
             }
@@ -305,154 +306,6 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
 
                 KeQueryTickCount( &pFcb->ObjectInformation->LastAccessCount);
 
-                pDirCB = pCcb->DirectoryCB;
-
-                //
-                // If this entry is deleted then remove the object from the volume tree
-                //
-
-                if( BooleanFlagOn( pDirCB->Flags, AFS_DIR_ENTRY_DELETED))
-                {
-
-                    if( pFcb->Header.NodeTypeCode == AFS_FILE_FCB)
-                    {
-
-                        //
-                        // Stop anything possibly in process
-                        //
-
-                        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
-                                      AFS_TRACE_LEVEL_VERBOSE,
-                                      "AFSClose Acquiring Fcb extents lock %08lX EXCL %08lX\n",
-                                      &pFcb->NPFcb->Specific.File.ExtentsResource,
-                                      PsGetCurrentThread());
-
-                        AFSAcquireExcl( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsResource,
-                                        TRUE);
-
-                        pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsRequestStatus = STATUS_FILE_DELETED;
-
-                        KeSetEvent( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsRequestComplete,
-                                    0,
-                                    FALSE);
-
-                        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
-                                      AFS_TRACE_LEVEL_VERBOSE,
-                                      "AFSClose Releasing Fcb extents lock %08lX EXCL %08lX\n",
-                                      &pFcb->NPFcb->Specific.File.ExtentsResource,
-                                      PsGetCurrentThread());
-
-                        AFSReleaseResource( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsResource);
-                    }
-
-                    AFSAcquireExcl( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock,
-                                    TRUE);
-
-                    AFSAcquireExcl( pObjectInfo->VolumeCB->ObjectInfoTree.TreeLock,
-                                    TRUE);
-
-                    if ( pDirCB->OpenReferenceCount == 0)
-                    {
-                        AFSDbgLogMsg( 0,
-                                      0,
-                                      "AFSClose (Other) OpenReferenceCount is Zero on DE %08lX Ccb %08lX FileName %wZ\n",
-                                      pDirCB,
-                                      pCcb,
-                                      &pDirCB->NameInformation.FileName);
-                    }
-
-                    ASSERT( pDirCB->OpenReferenceCount > 0);
-
-                    InterlockedDecrement( &pDirCB->OpenReferenceCount);
-
-                    AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
-                                  AFS_TRACE_LEVEL_VERBOSE,
-                                  "AFSClose (Other) Decrement count on %wZ DE %p Ccb %p Cnt %d\n",
-                                  &pDirCB->NameInformation.FileName,
-                                  pDirCB,
-                                  pCcb,
-                                  pDirCB->OpenReferenceCount);
-
-                    if( pDirCB->OpenReferenceCount == 0)
-                    {
-
-                        AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
-                                      AFS_TRACE_LEVEL_VERBOSE,
-                                      "AFSClose Deleting dir entry %08lX (%08lX) for %wZ  FID %08lX-%08lX-%08lX-%08lX\n",
-                                      pDirCB,
-                                      pObjectInfo,
-                                      &pDirCB->NameInformation.FileName,
-                                      pObjectInfo->FileId.Cell,
-                                      pObjectInfo->FileId.Volume,
-                                      pObjectInfo->FileId.Vnode,
-                                      pObjectInfo->FileId.Unique);
-
-                        //
-                        // Remove and delete the directory entry from the parent list
-                        //
-
-                        AFSDeleteDirEntry( pObjectInfo->ParentObjectInformation,
-                                           pDirCB);
-
-                        if( pObjectInfo->ObjectReferenceCount <= 0)
-                        {
-
-                            if( BooleanFlagOn( pObjectInfo->Flags, AFS_OBJECT_INSERTED_HASH_TREE))
-                            {
-
-                                AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
-                                              AFS_TRACE_LEVEL_VERBOSE,
-                                              "AFSClose Removing object %08lX from volume tree\n",
-                                              pObjectInfo);
-
-                                AFSRemoveHashEntry( &pObjectInfo->VolumeCB->ObjectInfoTree.TreeHead,
-                                                    &pObjectInfo->TreeEntry);
-
-                                ClearFlag( pObjectInfo->Flags, AFS_OBJECT_INSERTED_HASH_TREE);
-                            }
-
-                            SetFlag( pObjectInfo->Flags, AFS_OBJECT_FLAGS_DELETED);
-                        }
-                    }
-
-                    AFSReleaseResource( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock);
-
-                    AFSReleaseResource( pObjectInfo->VolumeCB->ObjectInfoTree.TreeLock);
-                }
-                else
-                {
-
-                    ASSERT( pDirCB->OpenReferenceCount > 0);
-
-                    InterlockedDecrement( &pDirCB->OpenReferenceCount);
-
-                    AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
-                                  AFS_TRACE_LEVEL_VERBOSE,
-                                  "AFSClose (Other2) Decrement count on %wZ DE %p Ccb %p Cnt %d\n",
-                                  &pDirCB->NameInformation.FileName,
-                                  pDirCB,
-                                  pCcb,
-                                  pDirCB->OpenReferenceCount);
-                }
-
-                //
-                // If this is not the root then decrement the open child reference count
-                //
-
-                if( pObjectInfo != NULL &&
-                    pObjectInfo->ParentObjectInformation != NULL &&
-                    pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount > 0)
-                {
-
-                    InterlockedDecrement( &pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
-
-                    AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
-                                  AFS_TRACE_LEVEL_VERBOSE,
-                                  "AFSClose Decrement child open ref count on Parent object %08lX Cnt %d\n",
-                                  pObjectInfo->ParentObjectInformation,
-                                  pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
-                }
-
                 if( pFcb->OpenReferenceCount == 1 &&
                     pFcb->Header.NodeTypeCode == AFS_FILE_FCB)
                 {
@@ -505,6 +358,8 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     AFSReleaseResource( &pFcb->NPFcb->Resource);
                 }
 
+                pDirCB = pCcb->DirectoryCB;
+
                 //
                 // Remove the Ccb and de-allocate it
                 //
@@ -527,19 +382,159 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     ntStatus = STATUS_SUCCESS;
                 }
 
+                //
+                // If this entry is deleted then remove the object from the volume tree
+                //
+
+                if( BooleanFlagOn( pDirCB->Flags, AFS_DIR_ENTRY_DELETED))
+                {
+
+                    if( pFcb->Header.NodeTypeCode == AFS_FILE_FCB &&
+                        pObjectInfo->Links == 0)
+                    {
+
+                        //
+                        // Stop anything possibly in process
+                        //
+
+                        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                                      AFS_TRACE_LEVEL_VERBOSE,
+                                      "AFSClose Acquiring Fcb extents lock %08lX EXCL %08lX\n",
+                                      &pFcb->NPFcb->Specific.File.ExtentsResource,
+                                      PsGetCurrentThread());
+
+                        AFSAcquireExcl( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsResource,
+                                        TRUE);
+
+                        pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsRequestStatus = STATUS_FILE_DELETED;
+
+                        KeSetEvent( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsRequestComplete,
+                                    0,
+                                    FALSE);
+
+                        AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
+                                      AFS_TRACE_LEVEL_VERBOSE,
+                                      "AFSClose Releasing Fcb extents lock %08lX EXCL %08lX\n",
+                                      &pFcb->NPFcb->Specific.File.ExtentsResource,
+                                      PsGetCurrentThread());
+
+                        AFSReleaseResource( &pObjectInfo->Fcb->NPFcb->Specific.File.ExtentsResource);
+                    }
+
+                    AFSAcquireExcl( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock,
+                                    TRUE);
+
+                    AFSAcquireExcl( pObjectInfo->VolumeCB->ObjectInfoTree.TreeLock,
+                                    TRUE);
+
+                    lCount = InterlockedDecrement( &pDirCB->DirOpenReferenceCount);
+
+                    AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
+                                  AFS_TRACE_LEVEL_VERBOSE,
+                                  "AFSClose (Other) Decrement count on %wZ DE %p Ccb %p Cnt %d\n",
+                                  &pDirCB->NameInformation.FileName,
+                                  pDirCB,
+                                  pCcb,
+                                  lCount);
+
+                    ASSERT( lCount >= 0);
+
+                    if( lCount == 0)
+                    {
+
+                        AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
+                                      AFS_TRACE_LEVEL_VERBOSE,
+                                      "AFSClose Deleting dir entry %08lX (%08lX) for %wZ  FID %08lX-%08lX-%08lX-%08lX\n",
+                                      pDirCB,
+                                      pObjectInfo,
+                                      &pDirCB->NameInformation.FileName,
+                                      pObjectInfo->FileId.Cell,
+                                      pObjectInfo->FileId.Volume,
+                                      pObjectInfo->FileId.Vnode,
+                                      pObjectInfo->FileId.Unique);
+
+                        //
+                        // Remove and delete the directory entry from the parent list
+                        //
+
+                        AFSDeleteDirEntry( pObjectInfo->ParentObjectInformation,
+                                           pDirCB);
+
+                        AFSAcquireShared( &pObjectInfo->NonPagedInfo->ObjectInfoLock,
+                                          TRUE);
+
+                        if( pObjectInfo->ObjectReferenceCount <= 0)
+                        {
+
+                            if( BooleanFlagOn( pObjectInfo->Flags, AFS_OBJECT_INSERTED_HASH_TREE))
+                            {
+
+                                AFSDbgLogMsg( AFS_SUBSYSTEM_CLEANUP_PROCESSING,
+                                              AFS_TRACE_LEVEL_VERBOSE,
+                                              "AFSClose Removing object %08lX from volume tree\n",
+                                              pObjectInfo);
+
+                                AFSRemoveHashEntry( &pObjectInfo->VolumeCB->ObjectInfoTree.TreeHead,
+                                                    &pObjectInfo->TreeEntry);
+
+                                ClearFlag( pObjectInfo->Flags, AFS_OBJECT_INSERTED_HASH_TREE);
+                            }
+                        }
+
+                        AFSReleaseResource( &pObjectInfo->NonPagedInfo->ObjectInfoLock);
+                    }
+
+                    AFSReleaseResource( pObjectInfo->ParentObjectInformation->Specific.Directory.DirectoryNodeHdr.TreeLock);
+
+                    AFSReleaseResource( pObjectInfo->VolumeCB->ObjectInfoTree.TreeLock);
+                }
+                else
+                {
+
+                    lCount = InterlockedDecrement( &pDirCB->DirOpenReferenceCount);
+
+                    AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
+                                  AFS_TRACE_LEVEL_VERBOSE,
+                                  "AFSClose (Other2) Decrement count on %wZ DE %p Ccb %p Cnt %d\n",
+                                  &pDirCB->NameInformation.FileName,
+                                  pDirCB,
+                                  pCcb,
+                                  lCount);
+
+                    ASSERT( lCount >= 0);
+                }
+
+                //
+                // If this is not the root then decrement the open child reference count
+                //
+
+                if( pObjectInfo != NULL &&
+                    pObjectInfo->ParentObjectInformation != NULL &&
+                    pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount > 0)
+                {
+
+                    InterlockedDecrement( &pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
+
+                    AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
+                                  AFS_TRACE_LEVEL_VERBOSE,
+                                  "AFSClose Decrement child open ref count on Parent object %08lX Cnt %d\n",
+                                  pObjectInfo->ParentObjectInformation,
+                                  pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
+                }
+
                 //
                 // Decrement the reference count on the Fcb. this is protecting it from teardown.
                 //
 
-                ASSERT( pFcb->OpenReferenceCount != 0);
-
-                InterlockedDecrement( &pFcb->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pFcb->OpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
                               "AFSClose Decrement count on Fcb %08lX Cnt %d\n",
                               pFcb,
-                              pFcb->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 break;
             }
@@ -549,6 +544,8 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
 
                 AFSPipeOpenCloseRequestCB stPipeClose;
 
+                pCcb = (AFSCcb *)pIrpSp->FileObject->FsContext2;
+
                 AFSDbgLogMsg( AFS_SUBSYSTEM_LOCK_PROCESSING,
                               AFS_TRACE_LEVEL_VERBOSE,
                               "AFSClose Acquiring Special Share lock %08lX EXCL %08lX\n",
@@ -558,10 +555,6 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                 AFSAcquireExcl( &pFcb->NPFcb->Resource,
                                 TRUE);
 
-                pCcb = (AFSCcb *)pIrpSp->FileObject->FsContext2;
-
-                pDirCB = pCcb->DirectoryCB;
-
                 RtlZeroMemory( &stPipeClose,
                                sizeof( AFSPipeOpenCloseRequestCB));
 
@@ -585,6 +578,8 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                                    NULL);
                 */
 
+                pDirCB = pCcb->DirectoryCB;
+
                 //
                 // Remove the Ccb and de-allocate it
                 //
@@ -606,9 +601,7 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     ntStatus = STATUS_SUCCESS;
                 }
 
-                ASSERT( pDirCB->OpenReferenceCount > 0);
-
-                InterlockedDecrement( &pDirCB->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pDirCB->DirOpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_DIRENTRY_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
@@ -616,7 +609,9 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                               &pDirCB->NameInformation.FileName,
                               pDirCB,
                               pCcb,
-                              pDirCB->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 //
                 // If this is not the root then decrement the open child reference count
@@ -626,26 +621,26 @@ AFSClose( IN PDEVICE_OBJECT LibDeviceObject,
                     pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount > 0)
                 {
 
-                    InterlockedDecrement( &pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
+                    lCount = InterlockedDecrement( &pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
 
                     AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                                   AFS_TRACE_LEVEL_VERBOSE,
                                   "AFSClose (Share) Decrement child open ref count on Parent object %08lX Cnt %d\n",
                                   pObjectInfo->ParentObjectInformation,
-                                  pObjectInfo->ParentObjectInformation->Specific.Directory.ChildOpenReferenceCount);
+                                  lCount);
                 }
 
                 AFSReleaseResource( &pFcb->NPFcb->Resource);
 
-                ASSERT( pFcb->OpenReferenceCount != 0);
-
-                InterlockedDecrement( &pFcb->OpenReferenceCount);
+                lCount = InterlockedDecrement( &pFcb->OpenReferenceCount);
 
                 AFSDbgLogMsg( AFS_SUBSYSTEM_FCB_REF_COUNTING,
                               AFS_TRACE_LEVEL_VERBOSE,
                               "AFSClose (Share) Decrement count on Fcb %08lX Cnt %d\n",
                               pFcb,
-                              pFcb->OpenReferenceCount);
+                              lCount);
+
+                ASSERT( lCount >= 0);
 
                 break;
             }
