@@ -42,9 +42,13 @@
  */
 
 #ifndef _WIN64
-#define HAVE_KRB4
+#define HAVE_KRB4 1
 #endif
 
+#include <afsconfig.h>
+#include <afs/param.h>
+#include <roken.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -79,10 +83,6 @@
 #include <afs\smb_iocons.h>
 #include <WINNT\afsreg.h>
 
-#define stat _stat
-#define __S_ISTYPE(mode, mask) (((mode) & _S_IFMT) == (mask))
-#define S_ISDIR(mode)          __S_ISTYPE((mode), _S_IFDIR)
-
 #define DONT_HAVE_GET_AD_TKT
 #define MAXSYMLINKS 255
 
@@ -197,8 +197,6 @@ static int use524 = FALSE;  /* use krb524? */
 static krb5_context context = 0;
 static krb5_ccache _krb425_ccache = 0;
 
-static krb5_error_code (KRB5_CALLCONV *pkrb5_init_context)(krb5_context *context) = NULL;
-static krb5_error_code (KRB5_CALLCONV *pkrb5_free_context)(krb5_context context) = NULL;
 static char * (KRB5_CALLCONV *pkrb5_get_error_message)(krb5_context context, krb5_error_code code)=NULL;
 static void (KRB5_CALLCONV *pkrb5_free_error_message)(krb5_context context, char *s) = NULL;
 
@@ -219,8 +217,6 @@ void akexit(int exit_code)
 void
 redirect_errors(const char *who, afs_int32 code, const char *fmt, va_list ap)
 {
-    krb5_context context;
-
     if (who) {
         fputs(who, stderr);
         fputs(": ", stderr);
@@ -229,22 +225,16 @@ redirect_errors(const char *who, afs_int32 code, const char *fmt, va_list ap)
         int freestr = 0;
         char *str = (char *)afs_error_message(code);
         if (strncmp(str, "unknown", strlen(str)) == 0) {
-            if (pkrb5_init_context &&
-                pkrb5_get_error_message &&
-                !pkrb5_init_context(&context))
-            {
+            if (pkrb5_get_error_message) {
                 str = pkrb5_get_error_message(NULL, code);
                 freestr = 1;
-            } else {
+            } else
                 str = (char *)error_message(code);
-            }
         }
         fputs(str, stderr);
         fputs(" ", stderr);
-        if (freestr) {
+        if (freestr)
             pkrb5_free_error_message(NULL, str);
-            pkrb5_free_context(context);
-        }
     }
     if (fmt) {
         vfprintf(stderr, fmt, ap);
@@ -337,6 +327,9 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
 #ifdef ALLOW_REGISTER
         } else if (strcmp(realm_of_user, realm_of_cell) != 0) {
             int i;
+            int flags = 0;
+            char * smbname = NULL;
+
             if (dflag) {
                 printf("doing first-time registration of %s "
                         "at %s\n", username, cell_to_use);
@@ -352,7 +345,16 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
                     aclient->cell[i] = toupper(aclient->cell[i]);
             }
 
-            if ((*status = ktc_SetToken(aserver, atoken, aclient, 0))) {
+            smbname = getenv("AFS_SMBNAME");
+            if ( smbname ) {
+                strncpy(aclient->smbname, smbname, MAXKTCNAMELEN - 1);
+                aclient->smbname[MAXKTCNAMELEN - 1] = '\0';
+                flags = AFS_SETTOK_LOGON;
+                if (dflag)
+                    printf("obtaining tokens for %s\n", aclient->smbname);
+            }
+
+            if ((*status = ktc_SetToken(aserver, atoken, aclient, flags))) {
                 afs_com_err(progname, *status,
                              "while obtaining tokens for cell %s\n",
                              cell_to_use);
@@ -482,7 +484,6 @@ static int get_v5cred(krb5_context context,
     krb5_creds increds;
     krb5_error_code r;
     static krb5_principal client_principal = 0;
-    int passes = 0;
 
     if (client_principal) {
         krb5_free_principal(context, client_principal);
@@ -515,12 +516,7 @@ static int get_v5cred(krb5_context context,
 	/* Ask for DES since that is what V4 understands */
     increds.keyblock.enctype = ENCTYPE_DES_CBC_CRC;
 
-    do {
-        r = krb5_get_credentials(context, 0, _krb425_ccache, &increds, creds);
-        if (r == KRB5KRB_AP_ERR_REPEAT)
-            Sleep(1000);
-    } while(r == KRB5KRB_AP_ERR_REPEAT && passes++ < 2);
-
+    r = krb5_get_credentials(context, 0, _krb425_ccache, &increds, creds);
     if (r) {
         return((int)r);
     }
@@ -774,6 +770,8 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     struct afsconf_cell ak_cellconfig; /* General information about the cell */
     int i;
     int getLinkedCell = 0;
+    int flags = 0;
+    char * smbname = getenv("AFS_SMBNAME");
 
     /* try to avoid an expensive call to get_cellconfig */
     if (cell && ll_string_check(&authedcells, cell))
@@ -1042,11 +1040,12 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     }
 
     if (!force &&
-         !ktc_GetToken(&aserver, &btoken, sizeof(btoken), &aclient) &&
-         atoken.kvno == btoken.kvno &&
-         atoken.ticketLen == btoken.ticketLen &&
-         !memcmp(&atoken.sessionKey, &btoken.sessionKey, sizeof(atoken.sessionKey)) &&
-         !memcmp(atoken.ticket, btoken.ticket, atoken.ticketLen))
+        !smbname &&
+        !ktc_GetToken(&aserver, &btoken, sizeof(btoken), &aclient) &&
+        atoken.kvno == btoken.kvno &&
+        atoken.ticketLen == btoken.ticketLen &&
+        !memcmp(&atoken.sessionKey, &btoken.sessionKey, sizeof(atoken.sessionKey)) &&
+        !memcmp(atoken.ticket, btoken.ticket, atoken.ticketLen))
     {
         if (dflag)
             printf("Identical tokens already exist; skipping.\n");
@@ -1114,12 +1113,22 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
             aclient.cell[i] = toupper(aclient.cell[i]);
     }
 
-    if (dflag)
-        printf("Getting tokens.\n");
-    if (status = ktc_SetToken(&aserver, &atoken, &aclient, 0))
+    if ( smbname ) {
+        if (dflag)
+            printf("Setting tokens for %s.\n", smbname);
+
+        strncpy(aclient.smbname, smbname, MAXKTCNAMELEN - 1);
+        aclient.smbname[MAXKTCNAMELEN - 1] = '\0';
+        flags = AFS_SETTOK_LOGON;
+    } else {
+        if (dflag)
+            printf("Setting tokens.\n");
+    }
+
+    if (status = ktc_SetToken(&aserver, &atoken, &aclient, flags))
     {
         afs_com_err(progname, status,
-                     "while obtaining tokens for cell %s\n",
+                     "while setting token for cell %s\n",
                      cell_to_use);
         status = AKLOG_TOKEN;
     }
@@ -1432,8 +1441,6 @@ load_krb5_error_message_funcs(void)
 {
     HINSTANCE h = LoadLibrary(KRB5LIB);
     if (h) {
-        (FARPROC)pkrb5_init_context = GetProcAddress(h, "krb5_init_context");
-        (FARPROC)pkrb5_free_context = GetProcAddress(h, "krb5_free_context");
         (FARPROC)pkrb5_get_error_message = GetProcAddress(h, "krb5_get_error_message");
         (FARPROC)pkrb5_free_error_message = GetProcAddress(h, "krb5_free_error_message");
     }
