@@ -20,12 +20,8 @@
 
 #include <afsconfig.h>
 #include "afs/param.h"
-#ifdef AFS_FBSD70_ENV
 #include <sys/param.h>
 #include <sys/vnode.h>
-     void
-     vgonel(struct vnode *vp, struct thread *td);
-#endif
 
 
 #include "afs/sysincludes.h"	/* Standard vendor system headers */
@@ -34,40 +30,31 @@
 #include <vm/vm_object.h>
 #include <vm/vm_map.h>
 #include <limits.h>
-#include <float.h>
 
 /*
  * FreeBSD implementation notes:
  * Most of these operations require us to frob vm_objects.  Most
  * functions require that the object be locked (with VM_OBJECT_LOCK)
- * on entry and leave it locked on exit.  In order to get the
- * vm_object itself we call VOP_GETVOBJECT on the vnode; the
- * locking protocol requires that we do so with the heavy vnode lock
- * held and the vnode interlock unlocked, and it returns the same
- * way.
+ * on entry and leave it locked on exit.  The locking protocol
+ * requires that we access vp->v_object with the heavy vnode lock
+ * held and the vnode interlock unlocked.
  *
  * The locking protocol for vnodes is defined in
- * kern/vnode_if.src and sys/vnode.h; the locking is still a work in 
- * progress, so some fields are (as of 5.1) still protected by Giant
- * rather than an explicit lock.
+ * kern/vnode_if.src and sys/vnode.h; unfortunately, it is not *quite*
+ * constant from version to version so to be properly correct we must
+ * check the VCS history of those files.
  */
 
-#ifdef AFS_FBSD60_ENV
-#define VOP_GETVOBJECT(vp, objp) (*(objp) = (vp)->v_object)
-#endif
-
 #if defined(AFS_FBSD80_ENV)
-#define	lock_vnode(v)	vn_lock((v), LK_EXCLUSIVE | LK_RETRY)
+#define	lock_vnode(v, f)	vn_lock((v), (f))
+#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY);
 #define unlock_vnode(v)	VOP_UNLOCK((v), 0)
-#elif defined(AFS_FBSD50_ENV)
-#define	lock_vnode(v)	vn_lock((v), LK_EXCLUSIVE | LK_RETRY, curthread)
-#define unlock_vnode(v)	VOP_UNLOCK((v), 0, curthread)
+#define islocked_vnode(v)	VOP_ISLOCKED((v))
 #else
-#define	lock_vnode(v)	vn_lock((v), LK_EXCLUSIVE | LK_RETRY, curproc)
-#define unlock_vnode(v)	VOP_UNLOCK((v), 0, curproc)
-/* need splvm() protection? */
-#define	VM_OBJECT_LOCK(o)
-#define VM_OBJECT_UNLOCK(o)
+#define	lock_vnode(v, f)	vn_lock((v), (f), curthread)
+#define ilock_vnode(v)	vn_lock((v), LK_INTERLOCK|LK_EXCLUSIVE|LK_RETRY, curthread);
+#define unlock_vnode(v)	VOP_UNLOCK((v), 0, curthread)
+#define islocked_vnode(v)	VOP_ISLOCKED((v), curthread)
 #endif
 
 /* Try to discard pages, in order to recycle a vcache entry.
@@ -84,52 +71,52 @@
  * is not dropped and re-acquired for any platform.  It may be that *slept is
  * therefore obsolescent.
  *
- * OSF/1 Locking:  VN_LOCK has been called.
- * We do not lock the vnode here, but instead require that it be exclusive
- * locked by code calling osi_VM_StoreAllSegments directly, or scheduling it
- * from the bqueue - Matt
- * Maybe better to just call vnode_pager_setsize()?
  */
 int
 osi_VM_FlushVCache(struct vcache *avc, int *slept)
 {
-    struct vm_object *obj;
-    struct vnode *vp;
-    if (VREFCOUNT(avc) > 1)
+    struct vnode *vp = AFSTOV(avc);
+
+    if (!VI_TRYLOCK(vp)) /* need interlock to check usecount */
 	return EBUSY;
 
-    if (avc->opens)
+    if (vp->v_usecount > 0) {
+	VI_UNLOCK(vp);
 	return EBUSY;
+    }
+
+    /* XXX
+     * The value of avc->opens here came to be, at some point,
+     * typically -1.  This was caused by incorrectly performing afs_close
+     * processing on vnodes being recycled */
+    if (avc->opens) {
+	VI_UNLOCK(vp);
+	return EBUSY;
+    }
 
     /* if a lock is held, give up */
-    if (CheckLock(&avc->lock))
+    if (CheckLock(&avc->lock)) {
+	VI_UNLOCK(vp);
 	return EBUSY;
-
-    return(0);
-
-    AFS_GUNLOCK();
-    vp = AFSTOV(avc);
-#ifndef AFS_FBSD70_ENV
-    lock_vnode(vp);
-#endif
-    if (VOP_GETVOBJECT(vp, &obj) == 0) {
-	VM_OBJECT_LOCK(obj);
-	vm_object_page_remove(obj, 0, 0, FALSE);
-#if 1
-	if (obj->ref_count == 0) {
-	    simple_lock(&vp->v_interlock);
-	    vgonel(vp, curthread);
-	    vp->v_tag = VT_AFS;
-	    SetAfsVnode(vp);
-	}
-#endif
-	VM_OBJECT_UNLOCK(obj);
     }
-#ifndef AFS_FBSD70_ENV
-    unlock_vnode(vp);
-#endif
-    AFS_GLOCK();
 
+    if ((vp->v_iflag & VI_DOOMED) != 0) {
+	VI_UNLOCK(vp);
+	return (0);
+    }
+
+    /* must hold the vnode before calling vgone()
+     * This code largely copied from vfs_subr.c:vlrureclaim() */
+    vholdl(vp);
+    AFS_GUNLOCK();
+    *slept = 1;
+    /* use the interlock while locking, so no one else can DOOM this */
+    ilock_vnode(vp);
+    vgone(vp);
+    unlock_vnode(vp);
+    vdrop(vp);
+
+    AFS_GLOCK();
     return 0;
 }
 
@@ -161,13 +148,12 @@ osi_VM_StoreAllSegments(struct vcache *avc)
      */
     do {
 	anyio = 0;
-	if (VOP_GETVOBJECT(vp, &obj) == 0 && (obj->flags & OBJ_MIGHTBEDIRTY)) {
-#ifdef AFS_FBSD50_ENV
+	
+	obj = vp->v_object;
+	if (obj != NULL && obj->flags & OBJ_MIGHTBEDIRTY) {
 	    if (!vget(vp, LK_EXCLUSIVE | LK_RETRY, curthread)) {
-#else
-		if (!vget(vp, LK_EXCLUSIVE | LK_RETRY | LK_NOOBJ, curproc)) {
-#endif
-		    if (VOP_GETVOBJECT(vp, &obj) == 0) {
+		    obj = vp->v_object;
+		    if (obj != NULL) {
 			VM_OBJECT_LOCK(obj);
 			vm_object_page_clean(obj, 0, 0, OBJPC_SYNC);
 			VM_OBJECT_UNLOCK(obj);
@@ -195,43 +181,57 @@ osi_VM_TryToSmush(struct vcache *avc, afs_ucred_t *acred, int sync)
 {
     struct vnode *vp;
     int tries, code;
-
-    SPLVAR;
+    int islocked;
 
     vp = AFSTOV(avc);
 
+    VI_LOCK(vp);
     if (vp->v_iflag & VI_DOOMED) {
-      USERPRI;
-      return;
+	VI_UNLOCK(vp);
+	return;
     }
+    VI_UNLOCK(vp);
+
+    islocked = islocked_vnode(vp);
+    if (islocked == LK_EXCLOTHER)
+	panic("Trying to Smush over someone else's lock");
+    else if (islocked == LK_SHARED) {
+	afs_warn("Trying to Smush with a shared lock");
+	lock_vnode(vp, LK_UPGRADE);
+    } else if (!islocked)
+	lock_vnode(vp, LK_EXCLUSIVE);
 
     if (vp->v_bufobj.bo_object != NULL) {
-      VM_OBJECT_LOCK(vp->v_bufobj.bo_object);
-      /*
-       * Do we really want OBJPC_SYNC?  OBJPC_INVAL would be
-       * faster, if invalidation is really what we are being
-       * asked to do.  (It would make more sense, too, since
-       * otherwise this function is practically identical to
-       * osi_VM_StoreAllSegments().)  -GAW
-       */
+	VM_OBJECT_LOCK(vp->v_bufobj.bo_object);
+	/*
+	 * Do we really want OBJPC_SYNC?  OBJPC_INVAL would be
+	 * faster, if invalidation is really what we are being
+	 * asked to do.  (It would make more sense, too, since
+	 * otherwise this function is practically identical to
+	 * osi_VM_StoreAllSegments().)  -GAW
+	 */
 
-      /*
-       * Dunno.  We no longer resemble osi_VM_StoreAllSegments,
-       * though maybe that's wrong, now.  And OBJPC_SYNC is the
-       * common thing in 70 file systems, it seems.  Matt.
-       */
+	/*
+	 * Dunno.  We no longer resemble osi_VM_StoreAllSegments,
+	 * though maybe that's wrong, now.  And OBJPC_SYNC is the
+	 * common thing in 70 file systems, it seems.  Matt.
+	 */
 
-      vm_object_page_clean(vp->v_bufobj.bo_object, 0, 0, OBJPC_SYNC);
-      VM_OBJECT_UNLOCK(vp->v_bufobj.bo_object);
+	vm_object_page_clean(vp->v_bufobj.bo_object, 0, 0, OBJPC_SYNC);
+	VM_OBJECT_UNLOCK(vp->v_bufobj.bo_object);
     }
 
     tries = 5;
     code = osi_vinvalbuf(vp, V_SAVE, PCATCH, 0);
     while (code && (tries > 0)) {
-      code = osi_vinvalbuf(vp, V_SAVE, PCATCH, 0);
-      --tries;
+	afs_warn("TryToSmush retrying vinvalbuf");
+	code = osi_vinvalbuf(vp, V_SAVE, PCATCH, 0);
+	--tries;
     }
-    USERPRI;
+    if (islocked == LK_SHARED)
+	lock_vnode(vp, LK_DOWNGRADE);
+    else if (!islocked)
+	unlock_vnode(vp);
 }
 
 /* Purge VM for a file when its callback is revoked.
@@ -246,12 +246,13 @@ osi_VM_FlushPages(struct vcache *avc, afs_ucred_t *credp)
 
     vp = AFSTOV(avc);
     ASSERT_VOP_LOCKED(vp, __func__);
-    if (VOP_GETVOBJECT(vp, &obj) == 0) {
+    obj = vp->v_object;
+    if (obj != NULL) {
 	VM_OBJECT_LOCK(obj);
 	vm_object_page_remove(obj, 0, 0, FALSE);
 	VM_OBJECT_UNLOCK(obj);
     }
-    /*vinvalbuf(AFSTOV(avc),0, NOCRED, curproc, 0,0); */
+    osi_vinvalbuf(vp, 0, 0, 0);
 }
 
 /* Purge pages beyond end-of-file, when truncating a file.
