@@ -35,34 +35,16 @@
  */
 
 #include <afsconfig.h>
-
-#include <stdio.h>
-#include <string.h>
-#include <ctype.h>
-#include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_MEMORY_H
-#include <memory.h>
-#endif
-#include <errno.h>
-
-#include <sys/stat.h>
-#include <fcntl.h>
-
-#include <sys/param.h>
-#include <sys/errno.h>
-#include <netdb.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <pwd.h>
-
+#include <afs/param.h>
 #include <afs/stds.h>
+
+#include <roken.h>
+
+#include <ctype.h>
+
+#include <afs/ktc.h>
+#include <afs/token.h>
+
 #include <krb5.h>
 #if defined(HAVE_ET_COM_ERR_H)
 #include <et/com_err.h>
@@ -74,14 +56,10 @@
 #include <afs/com_err.h>
 #endif
 
-#include <afs/param.h>
 #ifdef AFS_SUN5_ENV
 #include <sys/ioccom.h>
 #endif
 
-/* Prevent inclusion of des.h to avoid conflicts with des types */
-#define NO_DES_H_INCLUDE
-
 #include <afs/auth.h>
 #include <afs/cellconfig.h>
 #include <afs/vice.h>
@@ -119,10 +97,6 @@
 #define AKLOG_BADPATH 6
 #define AKLOG_MISC 7
 
-#ifndef NULL
-#define NULL 0
-#endif
-
 #ifndef TRUE
 #define TRUE 1
 #endif
@@ -242,11 +216,12 @@ Done:
 #error "Must have either keyblock or session member of krb5_creds"
 #endif
 
-/* MITKerberosShim logs but returns success */
-#if !defined(HAVE_KRB5_524_CONV_PRINCIPAL) || defined(AFS_DARWIN110_ENV) || (!defined(HAVE_KRB5_524_CONVERT_CREDS) && !defined(HAVE_KRB524_CONVERT_CREDS_KDC))
-#define HAVE_NO_KRB5_524
+#ifdef AFS_DARWIN110_ENV
+#define HAVE_NO_KRB5_524 /* MITKerberosShim logs but returns success */
 #elif !defined(HAVE_KRB5_524_CONVERT_CREDS) && defined(HAVE_KRB524_CONVERT_CREDS_KDC)
 #define krb5_524_convert_creds krb524_convert_creds_kdc
+#elif !defined(HAVE_KRB5_524_CONVERT_CREDS) && !defined(HAVE_KRB524_CONVERT_CREDS_KDC)
+#define HAVE_NO_KRB5_524
 #endif
 
 #if USING_HEIMDAL
@@ -378,7 +353,8 @@ copy_cellinfo(cellinfo_t *cellinfo)
 
 
 static int
-get_cellconfig(char *cell, struct afsconf_cell *cellconfig, char **local_cell)
+get_cellconfig(const char *config, char *cell,
+	       struct afsconf_cell *cellconfig, char **local_cell)
 {
     int status = AKLOG_SUCCESS;
     struct afsconf_dir *configdir;
@@ -392,10 +368,10 @@ get_cellconfig(char *cell, struct afsconf_cell *cellconfig, char **local_cell)
 	exit(AKLOG_AFS);
     }
 
-    if (!(configdir = afsconf_Open(AFSDIR_CLIENT_ETC_DIRPATH))) {
+    if (!(configdir = afsconf_Open(config))) {
 	fprintf(stderr,
 		"%s: can't get afs configuration (afsconf_Open(%s))\n",
-		progname, AFSDIR_CLIENT_ETC_DIRPATH);
+		progname, config);
 	exit(AKLOG_AFS);
     }
 
@@ -622,7 +598,7 @@ rxkad_get_ticket(krb5_context context, char *realm,
 	afs_com_err(progname, status, "while getting AFS tickets");
 #ifdef KRB5_CC_NOT_KTYPE
 	if (status == KRB5_CC_NOT_KTYPE) {
-	    fprintf(stderr, "allow_weak_crypto may be required in the Kerberos configuration\n");
+	    fprintf(stderr, "allow_weak_enctypes may be required in the Kerberos configuration\n");
 	}
 #endif
 	status = AKLOG_KERBEROS;
@@ -679,14 +655,14 @@ out:
  */
 static int
 rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
-			 struct ktc_token **tokenPtr, char **userPtr) {
+			 struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     char username[BUFSIZ];
-    struct ktc_token *token;
+    struct ktc_token token;
+    int status;
 #ifdef HAVE_NO_KRB5_524
     char *p;
     int len;
 #else
-    int status;
     char k4name[ANAME_SZ];
     char k4inst[INST_SZ];
     char k4realm[REALM_SZ];
@@ -729,21 +705,21 @@ rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
     }
 #endif
 
-    token = malloc(sizeof(struct ktc_token));
-    if (token == NULL)
-	return ENOMEM;
+    memset(&token, 0, sizeof(struct ktc_token));
 
-    memset(token, 0, sizeof(struct ktc_token));
-
-    token->kvno = RXKAD_TKT_TYPE_KERBEROS_V5;
-    token->startTime = v5cred->times.starttime;;
-    token->endTime = v5cred->times.endtime;
-    memcpy(&token->sessionKey, get_cred_keydata(v5cred),
+    token.kvno = RXKAD_TKT_TYPE_KERBEROS_V5;
+    token.startTime = v5cred->times.starttime;;
+    token.endTime = v5cred->times.endtime;
+    memcpy(&token.sessionKey, get_cred_keydata(v5cred),
 	   get_cred_keylen(v5cred));
-    token->ticketLen = v5cred->ticket.length;
-    memcpy(token->ticket, v5cred->ticket.data, token->ticketLen);
+    token.ticketLen = v5cred->ticket.length;
+    memcpy(token.ticket, v5cred->ticket.data, token.ticketLen);
+
+    status = token_importRxkadViceId(tokenPtr, &token, 0);
+    if (status) {
+	return status;
+    }
 
-    *tokenPtr = token;
     *userPtr = strdup(username);
 
     return 0;
@@ -773,7 +749,7 @@ rxkad_build_native_token(krb5_context context, krb5_creds *v5cred,
 #ifdef HAVE_NO_KRB5_524
 static int
 rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
-			  struct ktc_token **tokenPtr, char **userPtr) {
+			  struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     *tokenPtr = NULL;
     *userPtr = NULL;
 
@@ -782,10 +758,10 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
 #else
 static int
 rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
-			  struct ktc_token **tokenPtr, char **userPtr) {
+			  struct ktc_tokenUnion **tokenPtr, char **userPtr) {
     CREDENTIALS cred;
     char username[BUFSIZ];
-    struct ktc_token *token;
+    struct ktc_token token;
     int status;
 
     *tokenPtr = NULL;
@@ -807,11 +783,10 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
 	strcat (username, cred.pinst);
     }
 
-    token = malloc(sizeof(struct ktc_token));
-    memset(token, 0, sizeof(struct ktc_token));
+    memset(&token, 0, sizeof(struct ktc_token));
 
-    token->kvno = cred.kvno;
-    token->startTime = cred.issue_date;
+    token.kvno = cred.kvno;
+    token.startTime = cred.issue_date;
     /*
      * It seems silly to go through a bunch of contortions to
      * extract the expiration time, when the v5 credentials already
@@ -820,12 +795,16 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
      * Note that this isn't a security hole, as the expiration time
      * is also contained in the encrypted token
      */
-    token->endTime = v5cred->times.endtime;
-    memcpy(&token->sessionKey, cred.session, 8);
-    token->ticketLen = cred.ticket_st.length;
-    memcpy(token->ticket, cred.ticket_st.dat, token->ticketLen);
+    token.endTime = v5cred->times.endtime;
+    memcpy(&token.sessionKey, cred.session, 8);
+    token.ticketLen = cred.ticket_st.length;
+    memcpy(token.ticket, cred.ticket_st.dat, token.ticketLen);
+
+    status = token_importRxkadViceId(tokenPtr, &token, 0);
+    if (status) {
+	return status;
+    }
 
-    *tokenPtr = token;
     *userPtr = strdup(username);
 
     return 0;
@@ -857,7 +836,7 @@ rxkad_get_converted_token(krb5_context context, krb5_creds *v5cred,
  */
 static int
 rxkad_get_token(krb5_context context, struct afsconf_cell *cell, char *realm,
-		struct ktc_token **token, char **authuser, int *foreign) {
+		struct ktc_tokenUnion **token, char **authuser, int *foreign) {
     krb5_creds *v5cred;
     char *realmUsed = NULL;
     char *username = NULL;
@@ -889,7 +868,7 @@ rxkad_get_token(krb5_context context, struct afsconf_cell *cell, char *realm,
     } else {
 	len = strlen(username)+strlen(realmUsed)+2;
 	*authuser = malloc(len);
-	afs_snprintf(*authuser, len, "%s@%s", username, realmUsed);
+	snprintf(*authuser, len, "%s@%s", username, realmUsed);
 	*foreign = 1;
     }
 
@@ -902,68 +881,14 @@ out:
     return status;
 }
 
-static int
-get_kernel_token(struct afsconf_cell *cell, struct ktc_token **tokenPtr) {
-    struct ktc_principal client, server;
-    struct ktc_token *token;
-    int ret;
-
-    *tokenPtr = NULL;
-
-    strncpy(server.name, AFSKEY, MAXKTCNAMELEN - 1);
-    strncpy(server.instance, AFSINST, MAXKTCNAMELEN - 1);
-    strncpy(server.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    token = malloc(sizeof(struct ktc_token));
-    if (token == NULL)
-	return ENOMEM;
-
-    memset(token, 0, sizeof(struct ktc_token));
-
-    ret = ktc_GetToken(&server, token, sizeof(struct ktc_token), &client);
-    if (ret) {
-	free(token);
-	return ret;
-    }
-
-    *tokenPtr = token;
-    return 0;
-}
-
-static int
-set_kernel_token(struct afsconf_cell *cell, char *username,
-		 struct ktc_token *token, int setpag)
-{
-    struct ktc_principal client, server;
-
-    strncpy(client.name, username, MAXKTCNAMELEN - 1);
-    strcpy(client.instance, "");
-    strncpy(client.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    strncpy(server.name, AFSKEY, MAXKTCNAMELEN - 1);
-    strncpy(server.instance, AFSINST, MAXKTCNAMELEN - 1);
-    strncpy(server.cell, cell->name, MAXKTCREALMLEN - 1);
-
-    return ktc_SetToken(&server, token, &client, setpag);
-}
-
-static int
-tokens_equal(struct ktc_token *tokenA, struct ktc_token *tokenB) {
-    return (tokenA != NULL && tokenB != NULL &&
-	    tokenA->kvno == tokenB->kvno &&
-	    tokenA->ticketLen == tokenB->ticketLen &&
-	    !memcmp(&tokenA->sessionKey, &tokenB->sessionKey,
-		    sizeof(tokenA->sessionKey)) &&
-	    !memcmp(tokenA->ticket, tokenB->ticket, tokenA->ticketLen));
-}
-
-/*
+/* 
  * Log to a cell.  If the cell has already been logged to, return without
  * doing anything.  Otherwise, log to it and mark that it has been logged
  * to.
  */
 static int
-auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
+auth_to_cell(krb5_context context, const char *config, 
+	     char *cell, char *realm, char **linkedcell)
 {
     int status = AKLOG_SUCCESS;
     int isForeign = 0;
@@ -971,12 +896,13 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
     afs_int32 viceId;		/* AFS uid of user */
 
     char *local_cell = NULL;
-    struct ktc_token *token;
-    struct ktc_token *btoken;
+    struct ktc_tokenUnion *rxkadToken = NULL;
+    struct ktc_setTokenData *token;
+    struct ktc_setTokenData *btoken = NULL;
     struct afsconf_cell cellconf;
 
     /* NULL or empty cell returns information on local cell */
-    if ((status = get_cellconfig(cell, &cellconf, &local_cell)))
+    if ((status = get_cellconfig(config, cell, &cellconf, &local_cell)))
 	return(status);
 
     if (linkedcell != NULL) {
@@ -1028,20 +954,38 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	afs_dprintf("Authenticating to cell %s (server %s).\n", cellconf.name,
 		cellconf.hostName[0]);
 
-	status = rxkad_get_token(context, &cellconf, realm, &token,
+	token = token_buildTokenJar(cellconf.name);
+	if (token == NULL) {
+	    status = ENOMEM;
+	    goto out;
+	}
+
+	status = rxkad_get_token(context, &cellconf, realm, &rxkadToken,
 				 &username, &isForeign);
 	if (status)
-	    return status;
+	    goto out;
 
+	/* We need to keep the token structure around so that we can stick
+	 * the viceId into it (once we know it) */
+	status = token_addToken(token, rxkadToken);
+	if (status) {
+	    afs_dprintf("Add Token failed with %d", status);
+	    goto out;
+	}
 
 	if (!force &&
-	    !get_kernel_token(&cellconf, &btoken) &&
-	    tokens_equal(token, btoken)) {
+	    ktc_GetTokenEx(cellconf.name, &btoken) == 0 &&
+	    token_SetsEquivalent(token, btoken)) {
+
+	    token_FreeSet(&btoken);
 	    afs_dprintf("Identical tokens already exist; skipping.\n");
 	    status = AKLOG_SUCCESS;
 	    goto out;
 	}
 
+	if (btoken)
+	    token_FreeSet(&btoken);
+
 #ifdef FORCE_NOPRDB
 	noprdb = 1;
 #endif
@@ -1073,7 +1017,7 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 			username, cellconf.name);
 		viceId = 0;
 
-		status = set_kernel_token(&cellconf, username, token, 0);
+		status = ktc_SetTokenEx(token);
 		if (status) {
 		    afs_com_err(progname, status,
 				"while obtaining tokens for cell %s",
@@ -1084,7 +1028,7 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 		/*
 		 * In case you're wondering, we don't need to change the
 		 * filename here because we're still connecting to the
-		 * same cell -- we're just using a different authentication
+		 * same cell -- we're just using a different authenticat ion
 		 * level
 		 */
 
@@ -1106,27 +1050,17 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	    }
 #endif /* ALLOW_REGISTER */
 
-	    /*
-	     * This is a crock, but it is Transarc's crock, so we have to play
-	     * along in order to get the functionality.  The way the afs id is
-	     * stored is as a string in the username field of the token.
-	     * Contrary to what you may think by looking at the code for
-	     * tokens, this hack (AFS ID %d) will not work if you change %d
-	     * to something else.
-	     */
-
 	    if ((status == 0) && (viceId != ANONYMOUSID)) {
-		free(username);
-		if (afs_asprintf(&username, "AFS ID %d", (int) viceId) < 0) {
-		    status = ENOMEM;
-		    username = NULL;
-		    goto out;
+		status = token_setRxkadViceId(rxkadToken, viceId);
+		if (status) {
+		    fprintf(stderr, "Error %d setting rxkad ViceId\n", status);
+		    status = AKLOG_SUCCESS;
+		} else {
+		    token_replaceToken(token, rxkadToken);
 		}
 	    }
 	}
 
-	afs_dprintf("Set username to %s\n", username);
-
 	afs_dprintf("Setting tokens. %s @ %s \n", username, cellconf.name);
 
 #ifndef AFS_AIX51_ENV
@@ -1137,9 +1071,10 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	 */
 	write(2,"",0); /* dummy write */
 #endif
-	status = set_kernel_token(&cellconf, username, token, afssetpag);
+	token_setPag(token, afssetpag);
+	status = ktc_SetTokenEx(token);
 	if (status) {
-	    afs_com_err(progname, status, "while obtaining tokens for cell %s",
+	    afs_com_err(progname, status, "while setting tokens for cell %s",
 			cellconf.name);
 	    status = AKLOG_TOKEN;
 	}
@@ -1148,6 +1083,10 @@ auth_to_cell(krb5_context context, char *cell, char *realm, char **linkedcell)
 	afs_dprintf("Noauth mode; not authenticating.\n");
 
 out:
+    if (rxkadToken) {
+	token_freeToken(&rxkadToken);
+    }
+
     if (local_cell)
 	free(local_cell);
     if (username)
@@ -1359,7 +1298,7 @@ add_hosts(char *file)
  * every cell it encounters along the way.
  */
 static int
-auth_to_path(krb5_context context, char *path)
+auth_to_path(krb5_context context, const char *config, char *path)
 {
     int status = AKLOG_SUCCESS;
     int auth_status = AKLOG_SUCCESS;
@@ -1409,7 +1348,8 @@ auth_to_path(krb5_context context, char *path)
 		add_hosts(pathtocheck);
 	    if ((endofcell = strchr(mountpoint, VOLMARKER))) {
 		*endofcell = '\0';
-		if ((auth_status = auth_to_cell(context, cell, NULL, NULL))) {
+		auth_status = auth_to_cell(context, config, cell, NULL, NULL);
+		if (auth_status) {
 		    if (status == AKLOG_SUCCESS)
 			status = auth_status;
 		    else if (status != auth_status)
@@ -1497,6 +1437,7 @@ main(int argc, char *argv[])
     linked_list paths;		/* List of paths to log to */
     ll_node *cur_node;
     char *linkedcell;
+    const char *config = AFSDIR_CLIENT_ETC_DIRPATH;
 
     memset(&cellinfo, 0, sizeof(cellinfo));
 
@@ -1525,7 +1466,7 @@ main(int argc, char *argv[])
 	char *defaultpath = "~/Library/Preferences/edu.mit.Kerberos:/Library/Preferences/edu.mit.Kerberos";
 #endif
 	filepath = getenv("KRB5_CONFIG");
-	afs_asprintf(&newpath, "%s:%s/krb5-weak.conf",
+	asprintf(&newpath, "%s:%s/krb5-weak.conf",
 		 filepath ? filepath : defaultpath,
 		 AFSDIR_CLIENT_ETC_DIRPATH);
 	setenv("KRB5_CONFIG", newpath, 1);
@@ -1613,7 +1554,12 @@ main(int argc, char *argv[])
 	    }
 	    else
 		usage();
-
+	else if (strcmp(argv[i], "-config") == 0)
+	    if (++i < argc) {
+		config = argv[i];
+	    }
+	    else
+		usage();
 	else if (argv[i][0] == '-')
 	    usage();
 	else if (!pmode && !cmode) {
@@ -1687,7 +1633,7 @@ main(int argc, char *argv[])
     if ((cells.nelements + paths.nelements) == 0) {
 	struct passwd *pwd;
 
-	status = auth_to_cell(context, NULL, NULL, &linkedcell);
+	status = auth_to_cell(context, config, NULL, NULL, &linkedcell);
 
 	/* If this cell is linked to a DCE cell, and user requested -linked,
 	 * get tokens for both. This is very useful when the AFS cell is
@@ -1696,7 +1642,7 @@ main(int argc, char *argv[])
 
 	if (!status && linked && linkedcell != NULL) {
 	    afs_dprintf("Linked cell: %s\n", linkedcell);
-	    status = auth_to_cell(context, linkedcell, NULL, NULL);
+	    status = auth_to_cell(context, config, linkedcell, NULL, NULL);
 	}
 	if (linkedcell) {
 	    free(linkedcell);
@@ -1730,7 +1676,7 @@ main(int argc, char *argv[])
 
 		    afs_dprintf("Found cell %s in %s.\n", fcell, xlog_path);
 
-		    auth_status = auth_to_cell(context, fcell, NULL, NULL);
+		    auth_status = auth_to_cell(context, config, fcell, NULL, NULL);
 		    if (status == AKLOG_SUCCESS)
 			status = auth_status;
 		    else
@@ -1743,14 +1689,16 @@ main(int argc, char *argv[])
 	/* Log to all cells in the cells list first */
 	for (cur_node = cells.first; cur_node; cur_node = cur_node->next) {
 	    memcpy((char *)&cellinfo, cur_node->data, sizeof(cellinfo));
-	    if ((status = auth_to_cell(context, cellinfo.cell, cellinfo.realm,
-				       &linkedcell)))
+	    status = auth_to_cell(context, config, cellinfo.cell,
+				  cellinfo.realm, &linkedcell);
+	    if (status) {
 		somethingswrong++;
-	    else {
+	    } else {
 		if (linked && linkedcell != NULL) {
 		    afs_dprintf("Linked cell: %s\n", linkedcell);
-		    if ((status = auth_to_cell(context, linkedcell,
-					       cellinfo.realm, NULL)))
+		    status = auth_to_cell(context, config, linkedcell,
+					  cellinfo.realm, NULL);
+		    if (status)
 			somethingswrong++;
 		}
 		if (linkedcell != NULL) {
@@ -1762,7 +1710,8 @@ main(int argc, char *argv[])
 
 	/* Then, log to all paths in the paths list */
 	for (cur_node = paths.first; cur_node; cur_node = cur_node->next) {
-	    if ((status = auth_to_path(context, cur_node->data)))
+	    status = auth_to_path(context, config, cur_node->data);
+	    if (status)
 		somethingswrong++;
 	}
 
