@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -14,21 +14,25 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/gtx/curses_test.c,v 1.4 2001/07/12 19:58:39 shadow Exp $");
+#include <roken.h>
 
 #if defined(AFS_HPUX110_ENV) && !defined(__HP_CURSES)
-#define __HP_CURSES
+# define __HP_CURSES
 #endif
 
-#include <curses.h>
+#if defined(HAVE_NCURSES_H)
+# include <ncurses.h>
+#elif defined(HAVE_CURSES_H)
+# include <curses.h>
+#endif
 
 #include "AFS_component_version_number.c"
 
 main(argc, argv)
-    int argc;
-    char **argv;
+     int argc;
+     char **argv;
 
-{ /*main*/
+{				/*main */
 
     WINDOW *mainscr;
     char str[128];
@@ -46,14 +50,13 @@ main(argc, argv)
     standend();
 
 #if 0
-    box
-    addstr("Enter a string and a number: ");
+    box addstr("Enter a string and a number: ");
     refresh();
     scanw(stdscr, "%s %d", str, &i);
     wprintw(stdscr, "String was '%s', number was %d\n", str, i);
     refresh();
-#endif 0
+#endif /* 0 */
 
     endwin();
 
-} /*main*/
+}				/*main */
