@@ -122,7 +122,9 @@ afscp_CellByName(const char *cellname, const char *realmname)
     memset(thecell, 0, sizeof(struct afscp_cell));
     strlcpy(thecell->name, cellname, sizeof(thecell->name));
     if (realmname != NULL) {
-	thecell->realm = strdup(realmname);
+	thecell->realm = malloc(strlen(realmname) + 1);
+	memset(thecell->realm, 0, strlen(realmname) + 1);
+	strlcpy(thecell->realm, realmname, strlen(realmname) + 1);
     } else {
 	thecell->realm = NULL;
     }
@@ -286,10 +288,11 @@ afscp_ServerById(struct afscp_cell *thecell, afsUUID * u)
 	}
 	thecell->fsservers = newlist;
     }
-    ret = calloc(1, sizeof(struct afscp_server));
+    ret = malloc(sizeof(struct afscp_server));
     if (ret == NULL) {
 	return NULL;
     }
+    memset(ret, 0, sizeof(struct afscp_server));
     thecell->fsservers[thecell->nservers] = ret;
     memmove(&ret->id, u, sizeof(afsUUID));
     ret->cell = thecell->id;
@@ -375,10 +378,11 @@ afscp_ServerByAddr(struct afscp_cell *thecell, afs_uint32 addr)
 	}
 	thecell->fsservers = newlist;
     }
-    ret = calloc(1, sizeof(struct afscp_server));
+    ret = malloc(sizeof(struct afscp_server));
     if (ret == NULL) {
 	return NULL;
     }
+    memset(ret, 0, sizeof(struct afscp_server));
     thecell->fsservers[thecell->nservers] = ret;
     ret->cell = thecell->id;
     memset(&uuid, 0, sizeof(uuid));
