@@ -43,7 +43,6 @@
 #else /* KERNEL */
 # include <roken.h>
 # include <assert.h>
-# include <afs/opr.h>
 # if defined(AFS_NT40_ENV)
 #  ifndef EWOULDBLOCK
 #   define EWOULDBLOCK WSAEWOULDBLOCK
@@ -58,10 +57,9 @@
 # include <sys/sysmacros.h>
 #endif
 
-#include <opr/queue.h>
-
 #include "rx.h"
 #include "rx_clock.h"
+#include "rx_queue.h"
 #include "rx_packet.h"
 #include "rx_atomic.h"
 #include "rx_globals.h"
@@ -83,7 +81,7 @@ static afs_uint32       rx_packet_id = 0;
 
 extern char cml_version_number[];
 
-static int AllocPacketBufs(int class, int num_pkts, struct opr_queue *q);
+static int AllocPacketBufs(int class, int num_pkts, struct rx_queue *q);
 
 static void rxi_SendDebugPacket(struct rx_packet *apacket, osi_socket asocket,
 				afs_uint32 ahost, short aport,
@@ -103,10 +101,10 @@ static void rxi_AdjustLocalPacketsTSFPQ(int num_keep_local,
 static void rxi_FreePacketNoLock(struct rx_packet *p);
 static int rxi_FreeDataBufsNoLock(struct rx_packet *p, afs_uint32 first);
 static int rxi_FreeDataBufsToQueue(struct rx_packet *p, afs_uint32 first,
-				   struct opr_queue * q);
+				   struct rx_queue * q);
 #endif
 
-extern struct opr_queue rx_idleServerQueue;
+extern struct rx_queue rx_idleServerQueue;
 
 /* some rules about packets:
  * 1.  When a packet is allocated, the final iov_buf contains room for
@@ -241,14 +239,14 @@ rx_SlowWritePacket(struct rx_packet * packet, int offset, int resid, char *in)
 }
 
 int
-rxi_AllocPackets(int class, int num_pkts, struct opr_queue * q)
+rxi_AllocPackets(int class, int num_pkts, struct rx_queue * q)
 {
-    struct opr_queue *c;
+    struct rx_packet *p, *np;
 
     num_pkts = AllocPacketBufs(class, num_pkts, q);
 
-    for (opr_queue_Scan(q, c)) {
-        RX_PACKET_IOV_FULLINIT(opr_queue_Entry(c, struct rx_packet, entry));
+    for (queue_Scan(q, p, np, rx_packet)) {
+        RX_PACKET_IOV_FULLINIT(p);
     }
 
     return num_pkts;
@@ -256,7 +254,7 @@ rxi_AllocPackets(int class, int num_pkts, struct opr_queue * q)
 
 #ifdef RX_ENABLE_TSFPQ
 static int
-AllocPacketBufs(int class, int num_pkts, struct opr_queue * q)
+AllocPacketBufs(int class, int num_pkts, struct rx_queue * q)
 {
     struct rx_ts_info_t * rx_ts_info;
     int transfer;
@@ -286,7 +284,7 @@ AllocPacketBufs(int class, int num_pkts, struct opr_queue * q)
 }
 #else /* RX_ENABLE_TSFPQ */
 static int
-AllocPacketBufs(int class, int num_pkts, struct opr_queue * q)
+AllocPacketBufs(int class, int num_pkts, struct rx_queue * q)
 {
     struct rx_packet *c;
     int i;
@@ -339,13 +337,13 @@ AllocPacketBufs(int class, int num_pkts, struct opr_queue * q)
     }
 #endif /* KERNEL */
 
-    for (i=0, c=opr_queue_First(&rx_freePacketQueue, struct rx_packet, entry);
+    for (i=0, c=queue_First(&rx_freePacketQueue, rx_packet);
 	 i < num_pkts;
-	 i++, c=opr_queue_Next(&c->entry, struct rx_packet, entry)) {
+	 i++, c=queue_Next(c, rx_packet)) {
         RX_FPQ_MARK_USED(c);
     }
 
-    opr_queue_SplitBeforeAppend(&rx_freePacketQueue, q, &c->entry);
+    queue_SplitBeforeAppend(&rx_freePacketQueue,q,c);
 
     rx_nFreePackets -= num_pkts;
 
@@ -365,25 +363,22 @@ AllocPacketBufs(int class, int num_pkts, struct opr_queue * q)
 #ifdef RX_ENABLE_TSFPQ
 /* num_pkts=0 means queue length is unknown */
 int
-rxi_FreePackets(int num_pkts, struct opr_queue * q)
+rxi_FreePackets(int num_pkts, struct rx_queue * q)
 {
     struct rx_ts_info_t * rx_ts_info;
-    struct opr_queue *cursor, *store;
+    struct rx_packet *c, *nc;
     SPLVAR;
 
     osi_Assert(num_pkts >= 0);
     RX_TS_INFO_GET(rx_ts_info);
 
     if (!num_pkts) {
-	for (opr_queue_ScanSafe(q, cursor, store)) {
-	    num_pkts++;
-	    rxi_FreeDataBufsTSFPQ(opr_queue_Entry(cursor, struct rx_packet, 
-				    		 entry), 2, 0);
+	for (queue_Scan(q, c, nc, rx_packet), num_pkts++) {
+	    rxi_FreeDataBufsTSFPQ(c, 2, 0);
 	}
     } else {
-	for (opr_queue_ScanSafe(q, cursor, store)) {
-	    rxi_FreeDataBufsTSFPQ(opr_queue_Entry(cursor, struct rx_packet, 
-				    		 entry), 2, 0);
+	for (queue_Scan(q, c, nc, rx_packet)) {
+	    rxi_FreeDataBufsTSFPQ(c, 2, 0);
 	}
     }
 
@@ -409,20 +404,18 @@ rxi_FreePackets(int num_pkts, struct opr_queue * q)
 #else /* RX_ENABLE_TSFPQ */
 /* num_pkts=0 means queue length is unknown */
 int
-rxi_FreePackets(int num_pkts, struct opr_queue *q)
+rxi_FreePackets(int num_pkts, struct rx_queue *q)
 {
-    struct opr_queue cbs;
-    struct opr_queue *cursor, *store;
+    struct rx_queue cbs;
+    struct rx_packet *p, *np;
     int qlen = 0;
     SPLVAR;
 
     osi_Assert(num_pkts >= 0);
-    opr_queue_Init(&cbs);
+    queue_Init(&cbs);
 
     if (!num_pkts) {
-        for (opr_queue_ScanSafe(q, cursor, store)) {
-	    struct rx_packet *p
-		= opr_queue_Entry(cursor, struct rx_packet, entry);
+        for (queue_Scan(q, p, np, rx_packet), num_pkts++) {
 	    if (p->niovecs > 2) {
 		qlen += rxi_FreeDataBufsToQueue(p, 2, &cbs);
 	    }
@@ -431,10 +424,7 @@ rxi_FreePackets(int num_pkts, struct opr_queue *q)
 	if (!num_pkts)
 	    return 0;
     } else {
-        for (opr_queue_ScanSafe(q, cursor, store)) {
-	    struct rx_packet *p
-		= opr_queue_Entry(cursor, struct rx_packet, entry);
-
+        for (queue_Scan(q, p, np, rx_packet)) {
 	    if (p->niovecs > 2) {
 		qlen += rxi_FreeDataBufsToQueue(p, 2, &cbs);
 	    }
@@ -443,7 +433,7 @@ rxi_FreePackets(int num_pkts, struct opr_queue *q)
     }
 
     if (qlen) {
-	opr_queue_SpliceAppend(q, &cbs);
+	queue_SpliceAppend(q, &cbs);
 	qlen += num_pkts;
     } else
 	qlen = num_pkts;
@@ -451,7 +441,7 @@ rxi_FreePackets(int num_pkts, struct opr_queue *q)
     NETPRI;
     MUTEX_ENTER(&rx_freePktQ_lock);
 
-    opr_queue_SpliceAppend(&rx_freePacketQueue, q);
+    queue_SpliceAppend(&rx_freePacketQueue, q);
     rx_nFreePackets += qlen;
 
     /* Wakeup anyone waiting for packets */
@@ -501,7 +491,8 @@ int
 rxi_AllocDataBuf(struct rx_packet *p, int nb, int class)
 {
     int i, nv;
-    struct opr_queue q, *cursor, *store;
+    struct rx_queue q;
+    struct rx_packet *cb, *ncb;
 
     /* compute the number of cbuf's we need */
     nv = nb / RX_CBUFFERSIZE;
@@ -513,19 +504,14 @@ rxi_AllocDataBuf(struct rx_packet *p, int nb, int class)
         return nb;
 
     /* allocate buffers */
-    opr_queue_Init(&q);
+    queue_Init(&q);
     nv = AllocPacketBufs(class, nv, &q);
 
     /* setup packet iovs */
-    i = p ->niovecs;
-    for (opr_queue_ScanSafe(&q, cursor, store)) {
-	struct rx_packet *cb
-	    = opr_queue_Entry(cursor, struct rx_packet, entry);
-
-        opr_queue_Remove(&cb->entry);
+    for (i = p->niovecs, queue_Scan(&q, cb, ncb, rx_packet), i++) {
+        queue_Remove(cb);
         p->wirevec[i].iov_base = (caddr_t) cb->localdata;
         p->wirevec[i].iov_len = RX_CBUFFERSIZE;
-	i++;
     }
 
     nb -= (nv * RX_CBUFFERSIZE);
@@ -546,7 +532,7 @@ rxi_MorePackets(int apackets)
     SPLVAR;
 
     getme = apackets * sizeof(struct rx_packet);
-    p = osi_Alloc(getme);
+    p = (struct rx_packet *)osi_Alloc(getme);
     osi_Assert(p);
 
     PIN(p, getme);		/* XXXXX */
@@ -600,7 +586,7 @@ rxi_MorePackets(int apackets)
     SPLVAR;
 
     getme = apackets * sizeof(struct rx_packet);
-    p = osi_Alloc(getme);
+    p = (struct rx_packet *)osi_Alloc(getme);
     osi_Assert(p);
 
     PIN(p, getme);		/* XXXXX */
@@ -615,7 +601,7 @@ rxi_MorePackets(int apackets)
 #endif
 	p->niovecs = 2;
 
-	opr_queue_Append(&rx_freePacketQueue, &p->entry);
+	queue_Append(&rx_freePacketQueue, p);
 #ifdef RXDEBUG_PACKET
         p->packetId = rx_packet_id++;
         p->allNextp = rx_mallocedP;
@@ -643,7 +629,7 @@ rxi_MorePacketsTSFPQ(int apackets, int flush_global, int num_keep_local)
     SPLVAR;
 
     getme = apackets * sizeof(struct rx_packet);
-    p = osi_Alloc(getme);
+    p = (struct rx_packet *)osi_Alloc(getme);
 
     PIN(p, getme);		/* XXXXX */
     memset(p, 0, getme);
@@ -705,7 +691,7 @@ rxi_MorePacketsNoLock(int apackets)
 	* ((rx_maxJumboRecvSize - RX_FIRSTBUFFERSIZE) / RX_CBUFFERSIZE);
     do {
         getme = apackets * sizeof(struct rx_packet);
-        p = osi_Alloc(getme);
+        p = (struct rx_packet *)osi_Alloc(getme);
 	if (p == NULL) {
             apackets -= apackets / 4;
             osi_Assert(apackets > 0);
@@ -725,7 +711,7 @@ rxi_MorePacketsNoLock(int apackets)
 #endif
 	p->niovecs = 2;
 
-	opr_queue_Append(&rx_freePacketQueue, &p->entry);
+	queue_Append(&rx_freePacketQueue, p);
 #ifdef RXDEBUG_PACKET
         p->packetId = rx_packet_id++;
         p->allNextp = rx_mallocedP;
@@ -826,7 +812,7 @@ rxi_FreePacketNoLock(struct rx_packet *p)
 
     RX_FPQ_MARK_FREE(p);
     rx_nFreePackets++;
-    opr_queue_Append(&rx_freePacketQueue, &p->entry);
+    queue_Append(&rx_freePacketQueue, p);
 }
 #endif /* RX_ENABLE_TSFPQ */
 
@@ -867,7 +853,7 @@ rxi_FreePacketTSFPQ(struct rx_packet *p, int flush_global)
  */
 #ifndef RX_ENABLE_TSFPQ
 static int
-rxi_FreeDataBufsToQueue(struct rx_packet *p, afs_uint32 first, struct opr_queue * q)
+rxi_FreeDataBufsToQueue(struct rx_packet *p, afs_uint32 first, struct rx_queue * q)
 {
     struct iovec *iov;
     struct rx_packet * cb;
@@ -879,7 +865,7 @@ rxi_FreeDataBufsToQueue(struct rx_packet *p, afs_uint32 first, struct opr_queue
 	    osi_Panic("rxi_FreeDataBufsToQueue: unexpected NULL iov");
 	cb = RX_CBUF_TO_PACKET(iov->iov_base, p);
 	RX_FPQ_MARK_FREE(cb);
-	opr_queue_Append(q, &cb->entry);
+	queue_Append(q, cb);
     }
     p->length = 0;
     p->niovecs = 0;
@@ -1144,13 +1130,13 @@ rxi_AllocPacketNoLock(int class)
 
     if (rx_stats_active)
         rx_atomic_inc(&rx_stats.packetRequests);
-    if (opr_queue_IsEmpty(&rx_ts_info->_FPQ.queue)) {
+    if (queue_IsEmpty(&rx_ts_info->_FPQ)) {
 
 #ifdef KERNEL
-        if (opr_queue_IsEmpty(&rx_freePacketQueue))
+        if (queue_IsEmpty(&rx_freePacketQueue))
 	    osi_Panic("rxi_AllocPacket error");
 #else /* KERNEL */
-        if (opr_queue_IsEmpty(&rx_freePacketQueue))
+        if (queue_IsEmpty(&rx_freePacketQueue))
 	    rxi_MorePacketsNoLock(rx_maxSendWindow);
 #endif /* KERNEL */
 
@@ -1206,16 +1192,16 @@ rxi_AllocPacketNoLock(int class)
         rx_atomic_inc(&rx_stats.packetRequests);
 
 #ifdef KERNEL
-    if (opr_queue_IsEmpty(&rx_freePacketQueue))
+    if (queue_IsEmpty(&rx_freePacketQueue))
 	osi_Panic("rxi_AllocPacket error");
 #else /* KERNEL */
-    if (opr_queue_IsEmpty(&rx_freePacketQueue))
+    if (queue_IsEmpty(&rx_freePacketQueue))
 	rxi_MorePacketsNoLock(rx_maxSendWindow);
 #endif /* KERNEL */
 
     rx_nFreePackets--;
-    p = opr_queue_First(&rx_freePacketQueue, struct rx_packet, entry);
-    opr_queue_Remove(&p->entry);
+    p = queue_First(&rx_freePacketQueue, rx_packet);
+    queue_Remove(p);
     RX_FPQ_MARK_USED(p);
 
     dpf(("Alloc %"AFS_PTR_FMT", class %d\n", p, class));
@@ -1241,16 +1227,16 @@ rxi_AllocPacketTSFPQ(int class, int pull_global)
 
     if (rx_stats_active)
         rx_atomic_inc(&rx_stats.packetRequests);
-    if (pull_global && opr_queue_IsEmpty(&rx_ts_info->_FPQ.queue)) {
+    if (pull_global && queue_IsEmpty(&rx_ts_info->_FPQ)) {
         MUTEX_ENTER(&rx_freePktQ_lock);
 
-        if (opr_queue_IsEmpty(&rx_freePacketQueue))
+        if (queue_IsEmpty(&rx_freePacketQueue))
 	    rxi_MorePacketsNoLock(rx_maxSendWindow);
 
 	RX_TS_FPQ_GTOL(rx_ts_info);
 
         MUTEX_EXIT(&rx_freePktQ_lock);
-    } else if (opr_queue_IsEmpty(&rx_ts_info->_FPQ.queue)) {
+    } else if (queue_IsEmpty(&rx_ts_info->_FPQ)) {
         return NULL;
     }
 
@@ -1761,6 +1747,7 @@ rxi_ReceiveDebugPacket(struct rx_packet *ap, osi_socket asocket,
 {
     struct rx_debugIn tin;
     afs_int32 tl;
+    struct rx_serverQueueEntry *np, *nqe;
 
     /*
      * Only respond to client-initiated Rx debug packets,
@@ -1798,7 +1785,8 @@ rxi_ReceiveDebugPacket(struct rx_packet *ap, osi_socket asocket,
 	    tstat.usedFDs = CountFDs(64);
 	    tstat.nWaiting = htonl(rx_atomic_read(&rx_nWaiting));
 	    tstat.nWaited = htonl(rx_atomic_read(&rx_nWaited));
-	    tstat.idleThreads = opr_queue_Count(&rx_idleServerQueue);
+	    queue_Count(&rx_idleServerQueue, np, nqe, rx_serverQueueEntry,
+			tstat.idleThreads);
 	    MUTEX_EXIT(&rx_serverPool_lock);
 	    tstat.idleThreads = htonl(tstat.idleThreads);
 	    tl = sizeof(struct rx_debugStats) - ap->length;
@@ -1859,11 +1847,11 @@ rxi_ReceiveDebugPacket(struct rx_packet *ap, osi_socket asocket,
 			    tconn.callNumber[j] = htonl(tc->callNumber[j]);
 			    if ((tcall = tc->call[j])) {
 				tconn.callState[j] = tcall->state;
-				tconn.callMode[j] = tcall->app.mode;
+				tconn.callMode[j] = tcall->mode;
 				tconn.callFlags[j] = tcall->flags;
-				if (!opr_queue_IsEmpty(&tcall->rq))
+				if (queue_IsNotEmpty(&tcall->rq))
 				    tconn.callOther[j] |= RX_OTHER_IN;
-				if (!opr_queue_IsEmpty(&tcall->tq))
+				if (queue_IsNotEmpty(&tcall->tq))
 				    tconn.callOther[j] |= RX_OTHER_OUT;
 			    } else
 				tconn.callState[j] = RX_STATE_NOTINIT;
@@ -2543,6 +2531,14 @@ rxi_SendRawAbort(osi_socket socket, afs_uint32 host, u_short port,
     theader.securityIndex = source->header.securityIndex;
     theader.cid = htonl(source->header.cid);
 
+    /*
+     * If the abort is being sent in response to a server initiated packet,
+     * set client_initiated in the abort to ensure it is not associated by
+     * the receiver with a connection in the opposite direction.
+     */
+    if ((source->header.flags & RX_CLIENT_INITIATED) != RX_CLIENT_INITIATED)
+        theader.flags |= RX_CLIENT_INITIATED;
+
     error = htonl(error);
 
     iov[0].iov_base = &theader;
