@@ -13,28 +13,20 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-
-
 #include <afs/stds.h>
-#include <sys/types.h>
+
+#include <roken.h>
+
 #if (defined(AFS_AIX_ENV) && defined(KERNEL) && !defined(UKERNEL)) || defined(AFS_AUX_ENV) || defined(AFS_SUN5_ENV)
 #include <sys/systm.h>
 #endif
-#include <time.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#include <string.h>
+
 #include <rx/rx.h>
 #include <rx/xdr.h>
-#include <des.h>
+#include <rx/rx_packet.h>
 #include <afs/afsutil.h>
-#include <des/stats.h>
-#ifdef HAVE_STDINT_H
-# include <stdint.h>
-#endif
+
+#include "stats.h"
 #include "private_data.h"
 #define XPRT_RXKAD_SERVER
 
@@ -146,6 +138,8 @@ rxkad_NewServerSecurityObject(rxkad_level level, void *get_key_rock,
     struct rxkad_sprivate *tsp;
     int size;
 
+    rxkad_Init();
+
     if (!get_key)
 	return 0;
 
@@ -176,13 +170,12 @@ int
 rxkad_CheckAuthentication(struct rx_securityClass *aobj,
 			  struct rx_connection *aconn)
 {
-    struct rxkad_sconn *sconn;
+    struct rxkad_sconn *sconn = rx_GetSecurityData(aconn);
 
     /* first make sure the object exists */
-    if (!aconn->securityData)
+    if (!sconn)
 	return RXKADINCONSISTENCY;
 
-    sconn = (struct rxkad_sconn *)aconn->securityData;
     return !sconn->authenticated;
 }
 
@@ -193,10 +186,9 @@ int
 rxkad_CreateChallenge(struct rx_securityClass *aobj,
 		      struct rx_connection *aconn)
 {
-    struct rxkad_sconn *sconn;
+    struct rxkad_sconn *sconn = rx_GetSecurityData(aconn);
     struct rxkad_sprivate *tsp;
 
-    sconn = (struct rxkad_sconn *)aconn->securityData;
     sconn->challengeID = get_random_int32();
     sconn->authenticated = 0;	/* conn unauth. 'til we hear back */
     /* initialize level from object's minimum acceptable level */
@@ -211,13 +203,12 @@ int
 rxkad_GetChallenge(struct rx_securityClass *aobj, struct rx_connection *aconn,
 		   struct rx_packet *apacket)
 {
-    struct rxkad_sconn *sconn;
+    struct rxkad_sconn *sconn = rx_GetSecurityData(aconn);
     char *challenge;
     int challengeSize;
     struct rxkad_v2Challenge c_v2;	/* version 2 */
     struct rxkad_oldChallenge c_old;	/* old style */
 
-    sconn = (struct rxkad_sconn *)aconn->securityData;
     if (rx_IsUsingPktCksum(aconn))
 	sconn->cksumSeen = 1;
 
@@ -274,7 +265,7 @@ rxkad_CheckResponse(struct rx_securityClass *aobj,
     unsigned int pos;
     struct rxkad_serverinfo *rock;
 
-    sconn = (struct rxkad_sconn *)aconn->securityData;
+    sconn = rx_GetSecurityData(aconn);
     tsp = (struct rxkad_sprivate *)aobj->privateData;
 
     if (sconn->cksumSeen) {
@@ -336,8 +327,7 @@ rxkad_CheckResponse(struct rx_securityClass *aobj,
 	    tkt_DecodeTicket5(tix, tlen, tsp->get_key, tsp->get_key_rock,
 			      kvno, client.name, client.instance, client.cell,
 			      &sessionkey, &host, &start, &end,
-			      tsp->flags & RXS_CONFIG_FLAGS_DISABLE_DOTCHECK,
-			      tsp->alt_decrypt);
+			      tsp->flags & RXS_CONFIG_FLAGS_DISABLE_DOTCHECK);
 	if (code)
 	    return code;
     }
@@ -443,7 +433,7 @@ rxkad_GetServerInfo(struct rx_connection * aconn, rxkad_level * level,
 {
     struct rxkad_sconn *sconn;
 
-    sconn = (struct rxkad_sconn *)aconn->securityData;
+    sconn = rx_GetSecurityData(aconn);
     if (sconn && sconn->authenticated && sconn->rock
 	&& (time(0) < sconn->expirationTime)) {
 	if (level)
@@ -485,13 +475,3 @@ afs_int32 rxkad_SetConfiguration(struct rx_securityClass *aobj,
     }
     return 0;
 }
-
-int rxkad_SetAltDecryptProc(struct rx_securityClass *aobj,
-			    rxkad_alt_decrypt_func alt_decrypt)
-{
-    struct rxkad_sprivate *private =
-    (struct rxkad_sprivate *)aobj->privateData;
-
-    private->alt_decrypt = alt_decrypt;
-    return 0;
-}
